@@ -1,4 +1,4 @@
-// Copyright (c) 2020 Red Hat, Inc.
+// Copyright (c) 2021 Red Hat, Inc.
 
 package config
 
@@ -7,7 +7,7 @@ import (
 	"reflect"
 	"testing"
 
-	observatoriumv1alpha1 "github.com/observatorium/operator/api/v1alpha1"
+	observatoriumv1alpha1 "github.com/open-cluster-management/observatorium-operator/api/v1alpha1"
 	configv1 "github.com/openshift/api/config/v1"
 	routev1 "github.com/openshift/api/route/v1"
 	fakeconfigclient "github.com/openshift/client-go/config/clientset/versioned/fake"
@@ -44,6 +44,17 @@ func TestReplaceImage(t *testing.T) {
 		expected    bool
 		cm          map[string]string
 	}{
+		{
+			annotations: map[string]string{
+				AnnotationKeyImageRepository: DefaultImgRepository,
+				"mco-test-tag":               "test",
+			},
+			name:      "Replace image for test purpose",
+			imageRepo: "test.org",
+			expected:  true,
+			cm:        nil,
+		},
+
 		{
 			annotations: map[string]string{
 				AnnotationKeyImageRepository: DefaultImgRepository,
@@ -321,8 +332,7 @@ func TestGenerateMonitoringCR(t *testing.T) {
 	mco := &mcov1beta1.MultiClusterObservability{
 		TypeMeta: metav1.TypeMeta{Kind: "MultiClusterObservability"},
 		ObjectMeta: metav1.ObjectMeta{
-			Namespace: "test",
-			Name:      "test",
+			Name: "test",
 			Annotations: map[string]string{
 				AnnotationKeyImageTagSuffix: "tag",
 			},
@@ -351,6 +361,10 @@ func TestGenerateMonitoringCR(t *testing.T) {
 		t.Errorf("NodeSelector (%v) is not the expected (non-nil)", mco.Spec.NodeSelector)
 	}
 
+	if len(mco.Spec.Tolerations) != 0 {
+		t.Errorf("Tolerations (%v) is not the expected ([])", mco.Spec.Tolerations)
+	}
+
 	if mco.Spec.StorageConfig.StatefulSetSize != DefaultStorageSize {
 		t.Errorf("StatefulSetSize (%v) is not the expected (%v)",
 			mco.Spec.StorageConfig.StatefulSetSize,
@@ -378,14 +392,13 @@ func TestGenerateMonitoringCR(t *testing.T) {
 func TestGenerateMonitoringCustomizedCR(t *testing.T) {
 	addonSpec := &mcov1beta1.ObservabilityAddonSpec{
 		EnableMetrics: true,
-		Interval:      60,
+		Interval:      30,
 	}
 
 	mco := &mcov1beta1.MultiClusterObservability{
 		TypeMeta: metav1.TypeMeta{Kind: "MultiClusterObservability"},
 		ObjectMeta: metav1.ObjectMeta{
-			Namespace: "test",
-			Name:      "test",
+			Name: "test",
 			Annotations: map[string]string{
 				"test": "test",
 			},
@@ -425,6 +438,10 @@ func TestGenerateMonitoringCustomizedCR(t *testing.T) {
 			mco.Spec.StorageConfig.StatefulSetSize, "1Gi")
 	}
 
+	if len(mco.Spec.Tolerations) != 0 {
+		t.Errorf("Tolerations (%v) is not the expected ([])", mco.Spec.Tolerations)
+	}
+
 	if mco.Spec.StorageConfig.StatefulSetStorageClass != "gp2" {
 		t.Errorf("StatefulSetStorageClass (%v) is not the expected (%v)",
 			mco.Spec.StorageConfig.StatefulSetStorageClass, "gp2")
@@ -520,3 +537,229 @@ func TestReadImageManifestConfigMap(t *testing.T) {
 		})
 	}
 }
+
+func Test_checkIsIBMCloud(t *testing.T) {
+	s := scheme.Scheme
+	nodeIBM := &corev1.Node{
+		Spec: corev1.NodeSpec{
+			ProviderID: "ibm",
+		},
+	}
+	nodeOther := &corev1.Node{}
+
+	type args struct {
+		client client.Client
+		name   string
+	}
+	caselist := []struct {
+		name    string
+		args    args
+		want    bool
+		wantErr bool
+	}{
+		{
+			name: "is normal ocp",
+			args: args{
+				client: fake.NewFakeClientWithScheme(s, []runtime.Object{nodeOther}...),
+				name:   "test-secret",
+			},
+			want:    false,
+			wantErr: false,
+		},
+		{
+			name: "is ibm",
+			args: args{
+				client: fake.NewFakeClientWithScheme(s, []runtime.Object{nodeIBM}...),
+				name:   "test-secret",
+			},
+			want:    true,
+			wantErr: false,
+		},
+	}
+	for _, c := range caselist {
+		t.Run(c.name, func(t *testing.T) {
+			got, err := CheckIsIBMCloud(c.args.client)
+			if (err != nil) != c.wantErr {
+				t.Errorf("checkIsIBMCloud() error = %v, wantErr %v", err, c.wantErr)
+				return
+			}
+			if !reflect.DeepEqual(got, c.want) {
+				t.Errorf("checkIsIBMCloud() = %v, want %v", got, c.want)
+			}
+		})
+	}
+}
+
+func TestGetOperandName(t *testing.T) {
+	caseList := []struct {
+		name          string
+		componentName string
+		prepare       func()
+		result        func() bool
+	}{
+		{
+			name:          "No Observatorium CR",
+			componentName: Alertmanager,
+			prepare: func() {
+				SetOperandNames(fake.NewFakeClientWithScheme(runtime.NewScheme()))
+			},
+			result: func() bool {
+				return GetOperandName(Alertmanager) == GetOperandNamePrefix()+"alertmanager"
+			},
+		},
+		{
+			name:          "Have Observatorium CR without ownerreference",
+			componentName: Alertmanager,
+			prepare: func() {
+				//clean the operandNames map
+				for k := range operandNames {
+					delete(operandNames, k)
+				}
+				mco := &mcov1beta1.MultiClusterObservability{
+					TypeMeta: metav1.TypeMeta{Kind: "MultiClusterObservability"},
+					ObjectMeta: metav1.ObjectMeta{
+						Name: GetDefaultCRName(),
+					},
+					Spec: mcov1beta1.MultiClusterObservabilitySpec{
+						StorageConfig: &mcov1beta1.StorageConfigObject{
+							MetricObjectStorage: &mcov1beta1.PreConfiguredStorage{
+								Key:  "test",
+								Name: "test",
+							},
+						},
+					},
+				}
+
+				observatorium := &observatoriumv1alpha1.Observatorium{
+					ObjectMeta: metav1.ObjectMeta{
+						Name:      GetOperandNamePrefix() + "-observatorium",
+						Namespace: GetDefaultNamespace(),
+					},
+				}
+
+				// Register operator types with the runtime scheme.
+				s := scheme.Scheme
+				mcov1beta1.SchemeBuilder.AddToScheme(s)
+				observatoriumv1alpha1.AddToScheme(s)
+				client := fake.NewFakeClientWithScheme(s, []runtime.Object{mco, observatorium}...)
+				SetMonitoringCRName(GetDefaultCRName())
+				SetOperandNames(client)
+			},
+			result: func() bool {
+				return GetOperandName(Alertmanager) == GetOperandNamePrefix()+Alertmanager &&
+					GetOperandName(Grafana) == GetOperandNamePrefix()+Grafana &&
+					GetOperandName(Observatorium) == GetOperandNamePrefix()+Observatorium
+			},
+		},
+		{
+			name:          "Have Observatorium CR (observability-observatorium) with ownerreference",
+			componentName: Alertmanager,
+			prepare: func() {
+				//clean the operandNames map
+				for k := range operandNames {
+					delete(operandNames, k)
+				}
+				mco := &mcov1beta1.MultiClusterObservability{
+					TypeMeta: metav1.TypeMeta{Kind: "MultiClusterObservability"},
+					ObjectMeta: metav1.ObjectMeta{
+						Name: GetDefaultCRName(),
+					},
+					Spec: mcov1beta1.MultiClusterObservabilitySpec{
+						StorageConfig: &mcov1beta1.StorageConfigObject{
+							MetricObjectStorage: &mcov1beta1.PreConfiguredStorage{
+								Key:  "test",
+								Name: "test",
+							},
+						},
+					},
+				}
+
+				observatorium := &observatoriumv1alpha1.Observatorium{
+					ObjectMeta: metav1.ObjectMeta{
+						Name:      GetOperandNamePrefix() + "observatorium",
+						Namespace: GetDefaultNamespace(),
+						OwnerReferences: []metav1.OwnerReference{
+							{
+								Kind: "MultiClusterObservability",
+								Name: GetDefaultCRName(),
+							},
+						},
+					},
+				}
+
+				// Register operator types with the runtime scheme.
+				s := scheme.Scheme
+				mcov1beta1.SchemeBuilder.AddToScheme(s)
+				observatoriumv1alpha1.AddToScheme(s)
+				client := fake.NewFakeClientWithScheme(s, []runtime.Object{mco, observatorium}...)
+
+				SetMonitoringCRName(GetDefaultCRName())
+				SetOperandNames(client)
+			},
+			result: func() bool {
+				return GetOperandName(Alertmanager) == Alertmanager &&
+					GetOperandName(Grafana) == Grafana &&
+					GetOperandName(Observatorium) == GetOperandNamePrefix()+Observatorium
+			},
+		},
+		{
+			name:          "Have Observatorium CR (observability) with ownerreference",
+			componentName: Alertmanager,
+			prepare: func() {
+				//clean the operandNames map
+				for k := range operandNames {
+					delete(operandNames, k)
+				}
+				mco := &mcov1beta1.MultiClusterObservability{
+					TypeMeta: metav1.TypeMeta{Kind: "MultiClusterObservability"},
+					ObjectMeta: metav1.ObjectMeta{
+						Name: GetDefaultCRName(),
+					},
+					Spec: mcov1beta1.MultiClusterObservabilitySpec{
+						StorageConfig: &mcov1beta1.StorageConfigObject{
+							MetricObjectStorage: &mcov1beta1.PreConfiguredStorage{
+								Key:  "test",
+								Name: "test",
+							},
+						},
+					},
+				}
+
+				observatorium := &observatoriumv1alpha1.Observatorium{
+					ObjectMeta: metav1.ObjectMeta{
+						Name:      GetDefaultCRName(),
+						Namespace: GetDefaultNamespace(),
+						OwnerReferences: []metav1.OwnerReference{
+							{
+								Kind: "MultiClusterObservability",
+								Name: GetDefaultCRName(),
+							},
+						},
+					},
+				}
+
+				// Register operator types with the runtime scheme.
+				s := scheme.Scheme
+				mcov1beta1.SchemeBuilder.AddToScheme(s)
+				observatoriumv1alpha1.AddToScheme(s)
+				client := fake.NewFakeClientWithScheme(s, []runtime.Object{mco, observatorium}...)
+
+				SetMonitoringCRName(GetDefaultCRName())
+				SetOperandNames(client)
+			},
+			result: func() bool {
+				return GetOperandName(Alertmanager) == GetOperandNamePrefix()+Alertmanager &&
+					GetOperandName(Grafana) == GetOperandNamePrefix()+Grafana &&
+					GetOperandName(Observatorium) == GetOperandNamePrefix()+Observatorium
+			},
+		},
+	}
+	for _, c := range caseList {
+		t.Run(c.name, func(t *testing.T) {
+			c.prepare()
+			if !c.result() {
+				t.Errorf("case (%v) output is not the expected", c.name)
+			}
+		})
+	}
+}
