@@ -1,4 +1,4 @@
-// Copyright (c) 2020 Red Hat, Inc.
+// Copyright (c) 2021 Red Hat, Inc.
 
 package multiclusterobservability
 
@@ -6,10 +6,9 @@ import (
 	"context"
 	"fmt"
 	"os"
-	"strings"
 	"time"
 
-	observatoriumv1alpha1 "github.com/observatorium/operator/api/v1alpha1"
+	observatoriumv1alpha1 "github.com/open-cluster-management/observatorium-operator/api/v1alpha1"
 	ocpClientSet "github.com/openshift/client-go/config/clientset/versioned"
 	appsv1 "k8s.io/api/apps/v1"
 	corev1 "k8s.io/api/core/v1"
@@ -36,11 +35,7 @@ import (
 )
 
 const (
-	certFinalizer   = "observability.open-cluster-management.io/cert-cleanup"
-	Installing      = "Installing"
-	Failed          = "Failed"
-	Ready           = "Ready"
-	MetricsDisabled = "Disabled"
+	certFinalizer = "observability.open-cluster-management.io/cert-cleanup"
 )
 
 var (
@@ -107,18 +102,54 @@ func add(mgr manager.Manager, r reconcile.Reconciler) error {
 		IsController: true,
 		OwnerType:    &mcov1beta1.MultiClusterObservability{},
 	})
+	if err != nil {
+		return err
+	}
 
 	// Watch for changes to secondary resource statefulSet and requeue the owner MultiClusterObservability
 	err = c.Watch(&source.Kind{Type: &appsv1.StatefulSet{}}, &handler.EnqueueRequestForOwner{
 		IsController: true,
 		OwnerType:    &mcov1beta1.MultiClusterObservability{},
 	})
+	if err != nil {
+		return err
+	}
 
 	// Watch for changes to secondary resource ConfigMap and requeue the owner MultiClusterObservability
 	err = c.Watch(&source.Kind{Type: &corev1.ConfigMap{}}, &handler.EnqueueRequestForOwner{
 		IsController: true,
 		OwnerType:    &mcov1beta1.MultiClusterObservability{},
 	})
+	if err != nil {
+		return err
+	}
+
+	// Watch for changes to secondary resource Secret and requeue the owner MultiClusterObservability
+	err = c.Watch(&source.Kind{Type: &corev1.Secret{}}, &handler.EnqueueRequestForOwner{
+		IsController: true,
+		OwnerType:    &mcov1beta1.MultiClusterObservability{},
+	})
+	if err != nil {
+		return err
+	}
+
+	// Watch for changes to secondary resource Service and requeue the owner MultiClusterObservability
+	err = c.Watch(&source.Kind{Type: &corev1.Service{}}, &handler.EnqueueRequestForOwner{
+		IsController: true,
+		OwnerType:    &mcov1beta1.MultiClusterObservability{},
+	})
+	if err != nil {
+		return err
+	}
+
+	// Watch for changes to secondary Observatorium CR and requeue the owner MultiClusterObservability
+	err = c.Watch(&source.Kind{Type: &observatoriumv1alpha1.Observatorium{}}, &handler.EnqueueRequestForOwner{
+		IsController: true,
+		OwnerType:    &mcov1beta1.MultiClusterObservability{},
+	})
+	if err != nil {
+		return err
+	}
 
 	pred = predicate.Funcs{
 		CreateFunc: func(e event.CreateEvent) bool {
@@ -174,28 +205,6 @@ func add(mgr manager.Manager, r reconcile.Reconciler) error {
 		return err
 	}
 
-	// Watch for changes to secondary resource Secret and requeue the owner MultiClusterObservability
-	err = c.Watch(&source.Kind{Type: &corev1.Secret{}}, &handler.EnqueueRequestForOwner{
-		IsController: true,
-		OwnerType:    &mcov1beta1.MultiClusterObservability{},
-	})
-
-	// Watch for changes to secondary resource Service and requeue the owner MultiClusterObservability
-	err = c.Watch(&source.Kind{Type: &corev1.Service{}}, &handler.EnqueueRequestForOwner{
-		IsController: true,
-		OwnerType:    &mcov1beta1.MultiClusterObservability{},
-	})
-
-	// Watch for changes to secondary Observatorium CR and requeue the owner MultiClusterObservability
-	err = c.Watch(&source.Kind{Type: &observatoriumv1alpha1.Observatorium{}}, &handler.EnqueueRequestForOwner{
-		IsController: true,
-		OwnerType:    &mcov1beta1.MultiClusterObservability{},
-	})
-
-	if err != nil {
-		return err
-	}
-
 	return nil
 }
 
@@ -267,7 +276,13 @@ func (r *ReconcileMultiClusterObservability) Reconcile(request reconcile.Request
 		return reconcile.Result{}, err
 	}
 
-	instance.Namespace = config.GetDefaultNamespace()
+	//set operand names to cover the upgrade case since we have name changed in new release
+	err = config.SetOperandNames(r.client)
+	if err != nil {
+		return reconcile.Result{}, err
+	}
+
+	//instance.Namespace = config.GetDefaultNamespace()
 	instance.Spec.StorageConfig.StatefulSetStorageClass = storageClassSelected
 	//Render the templates with a specified CR
 	renderer := rendering.NewRenderer(instance)
@@ -279,13 +294,14 @@ func (r *ReconcileMultiClusterObservability) Reconcile(request reconcile.Request
 	deployer := deploying.NewDeployer(r.client)
 	//Deploy the resources
 	for _, res := range toDeploy {
-		if res.GetNamespace() == instance.Namespace {
+		if res.GetNamespace() == config.GetDefaultNamespace() {
 			if err := controllerutil.SetControllerReference(instance, res, r.scheme); err != nil {
 				reqLogger.Error(err, "Failed to set controller reference")
 			}
 		}
 		if err := deployer.Deploy(res); err != nil {
-			reqLogger.Error(err, fmt.Sprintf("Failed to deploy %s %s/%s", res.GetKind(), instance.Namespace, res.GetName()))
+			reqLogger.Error(err, fmt.Sprintf("Failed to deploy %s %s/%s",
+				res.GetKind(), config.GetDefaultNamespace(), res.GetName()))
 			return reconcile.Result{}, err
 		}
 	}
@@ -334,114 +350,45 @@ func (r *ReconcileMultiClusterObservability) Reconcile(request reconcile.Request
 // UpdateStatus override UpdateStatus interface
 func (r *ReconcileMultiClusterObservability) UpdateStatus(
 	mco *mcov1beta1.MultiClusterObservability) (*reconcile.Result, error) {
-
-	reqLogger := log.WithValues("Request.Namespace", mco.Namespace, "Request.Name", mco.Name)
-	requeue := false
-
-	deployList := &appsv1.DeploymentList{}
-	deploymentListOpts := []client.ListOption{
-		client.InNamespace(mco.Namespace),
-		client.MatchingLabels(labelsForMultiClusterMonitoring(mco.Name)),
-	}
-
-	err := r.client.List(context.TODO(), deployList, deploymentListOpts...)
-	if err != nil {
-		reqLogger.Error(err, "Failed to list deployments.",
-			"MultiClusterObservability.Namespace", mco.Namespace,
-			"MemcaMultiClusterMonitoringched.Name", mco.Name,
-		)
-		return &reconcile.Result{}, err
-	}
-
-	installingCondition := CheckInstallStatus(r.client, mco)
-	conditions := []mcov1beta1.MCOCondition{}
-	allDeploymentReady := true
-	failedDeployment := ""
-	for _, deployment := range deployList.Items {
-		if deployment.Status.ReadyReplicas < 1 || deployment.Status.AvailableReplicas < 1 {
-			allDeploymentReady = false
-			failedDeployment = deployment.Name
-			break
-		}
-	}
-
-	// validate s3 conf
-	s3condition := CheckS3Conf(r.client, mco)
-	if s3condition != nil {
-		conditions = append(conditions, *s3condition)
-		requeue = true
-	} else {
-		if installingCondition.Type != "Ready" {
-			conditions = append(conditions, installingCondition)
-			requeue = true
-		} else if len(deployList.Items) == 0 {
-			conditions = append(conditions, mcov1beta1.MCOCondition{
-				Type:    Failed,
-				Reason:  Failed,
-				Message: "No deployment found.",
-			})
-			requeue = true
-		} else {
-			if allDeploymentReady {
-				ready := mcov1beta1.MCOCondition{}
-				requeue = false
-				if mco.Spec.ObservabilityAddonSpec != nil && mco.Spec.ObservabilityAddonSpec.EnableMetrics == false {
-					ready = mcov1beta1.MCOCondition{
-						Type:    Ready,
-						Reason:  Ready,
-						Message: "Observability components are deployed and running",
-					}
-					conditions = append(conditions, ready)
-					enableMetrics := mcov1beta1.MCOCondition{
-						Type:    MetricsDisabled,
-						Reason:  MetricsDisabled,
-						Message: "Collect metrics from the managed clusters is disabled",
-					}
-					conditions = append(conditions, enableMetrics)
-				} else {
-					ready = mcov1beta1.MCOCondition{
-						Type:    Ready,
-						Reason:  Ready,
-						Message: "Observability components are deployed and running",
-					}
-					conditions = append(conditions, ready)
-				}
-
-			} else {
-				failedMessage := fmt.Sprintf("Deployment is failed for %s", failedDeployment)
-				failed := mcov1beta1.MCOCondition{
-					Type:    Failed,
-					Reason:  Failed,
-					Message: failedMessage,
-				}
-				conditions = append(conditions, failed)
-				requeue = true
-			}
-		}
-	}
-
-	mco.Status.Conditions = conditions
-	err = r.client.Status().Update(context.TODO(), mco)
+	log.Info("Update MCO status")
+	oldStatus := &mco.Status
+	newStatus := oldStatus.DeepCopy()
+	updateInstallStatus(&newStatus.Conditions)
+	updateReadyStatus(&newStatus.Conditions, r.client, mco)
+	updateAddonSpecStatus(&newStatus.Conditions, mco)
+	fillupStatus(&newStatus.Conditions)
+	mco.Status.Conditions = newStatus.Conditions
+	err := r.client.Status().Update(context.TODO(), mco)
 	if err != nil {
 		if apierrors.IsConflict(err) {
 			// Error from object being modified is normal behavior and should not be treated like an error
 			log.Info("Failed to update status", "Reason", "Object has been modified")
-			return &reconcile.Result{RequeueAfter: time.Second}, nil
+			found := &mcov1beta1.MultiClusterObservability{}
+			err = r.client.Get(context.TODO(), types.NamespacedName{
+				Name: config.GetMonitoringCRName(),
+			}, found)
+			if err != nil {
+				log.Error(err, fmt.Sprintf("Failed to get existing mco %s", mco.Name))
+				return &reconcile.Result{}, err
+			}
+			mco.ObjectMeta.ResourceVersion = found.ObjectMeta.ResourceVersion
+			err = r.client.Status().Update(context.TODO(), mco)
+			if err != nil {
+				log.Error(err, fmt.Sprintf("Failed to update %s status ", mco.Name))
+				return &reconcile.Result{}, err
+			}
+			return &reconcile.Result{Requeue: true, RequeueAfter: time.Second}, nil
 		}
 
-		log.Error(err, fmt.Sprintf("Failed to update %s/%s status ", mco.Namespace, mco.Name))
+		log.Error(err, fmt.Sprintf("Failed to update %s status ", mco.Name))
 		return &reconcile.Result{}, err
 	}
-	if requeue {
-		return &reconcile.Result{RequeueAfter: time.Second * 2}, nil
-	}
-	return nil, nil
-}
 
-// labelsForMultiClusterMonitoring returns the labels for selecting the resources
-// belonging to the given MultiClusterObservability CR name.
-func labelsForMultiClusterMonitoring(name string) map[string]string {
-	return map[string]string{"observability.open-cluster-management.io/name": name}
+	if findStatusCondition(newStatus.Conditions, "Ready") == nil {
+		return &reconcile.Result{Requeue: true, RequeueAfter: time.Second * 2}, nil
+	}
+
+	return nil, nil
 }
 
 func (r *ReconcileMultiClusterObservability) initFinalization(
@@ -455,7 +402,7 @@ func (r *ReconcileMultiClusterObservability) initFinalization(
 		mco.SetFinalizers(util.Remove(mco.GetFinalizers(), certFinalizer))
 		err = r.client.Update(context.TODO(), mco)
 		if err != nil {
-			log.Error(err, "Failed to remove finalizer from mco resource", "namespace", mco.Namespace)
+			log.Error(err, "Failed to remove finalizer from mco resource")
 			return false, err
 		}
 		log.Info("Finalizer removed from mco resource")
@@ -465,7 +412,7 @@ func (r *ReconcileMultiClusterObservability) initFinalization(
 		mco.SetFinalizers(append(mco.GetFinalizers(), certFinalizer))
 		err := r.client.Update(context.TODO(), mco)
 		if err != nil {
-			log.Error(err, "Failed to add finalizer to mco resource", "namespace", mco.Namespace)
+			log.Error(err, "Failed to add finalizer to mco resource")
 			return false, err
 		}
 		log.Info("Finalizer added to mco resource")
@@ -473,155 +420,6 @@ func (r *ReconcileMultiClusterObservability) initFinalization(
 	return false, nil
 }
 
-func getResourceConditions(podList *corev1.PodList, statefulSetList *appsv1.StatefulSetList,
-	watchingPods []string, watchingStatefulSets []string,
-	allPodsReady *bool, allstatefulSetReady *bool,
-	podCounter *int, statefulSetCounter *int,
-) {
-	for _, pod := range podList.Items {
-		for _, name := range watchingPods {
-			if strings.HasPrefix(pod.Name, name) {
-				*podCounter++
-				singlePodReady := false
-				for _, podCondition := range pod.Status.Conditions {
-					if podCondition.Type == Ready {
-						singlePodReady = true
-					}
-				}
-				if singlePodReady == false {
-					*allPodsReady = false
-				}
-			}
-		}
-	}
-	for _, statefulSet := range statefulSetList.Items {
-		for _, name := range watchingStatefulSets {
-			if strings.HasPrefix(statefulSet.Name, name) {
-				*statefulSetCounter++
-				singleStatefulSetReady := false
-				if statefulSet.Status.ReadyReplicas >= 1 {
-					singleStatefulSetReady = true
-				}
-				if singleStatefulSetReady == false {
-					*allstatefulSetReady = false
-				}
-			}
-		}
-	}
-	return
-}
-
-func CheckInstallStatus(c client.Client,
-	mco *mcov1beta1.MultiClusterObservability) mcov1beta1.MCOCondition {
-
-	installingCondition := mcov1beta1.MCOCondition{}
-	if len(mco.Status.Conditions) == 0 {
-		installingCondition = mcov1beta1.MCOCondition{
-			Type:    Installing,
-			Reason:  Installing,
-			Message: "Installation is in progress",
-		}
-	} else if mco.Status.Conditions[0].Type == Installing {
-		installingCondition = mco.Status.Conditions[0]
-		watchingPods := []string{
-			strings.Join([]string{mco.ObjectMeta.Name, "observatorium-observatorium-api"}, "-"),
-			strings.Join([]string{mco.ObjectMeta.Name, "observatorium-thanos-query"}, "-"),
-			strings.Join([]string{mco.ObjectMeta.Name, "observatorium-thanos-receive-controller"}, "-"),
-			"grafana",
-			"alertmanager",
-		}
-		podList := &corev1.PodList{}
-		podListOpts := []client.ListOption{
-			client.InNamespace(mco.Namespace),
-		}
-		err := c.List(context.TODO(), podList, podListOpts...)
-		if err != nil {
-			log.Error(err, "Failed to list pods.",
-				"MultiClusterObservability.Namespace", mco.Namespace,
-			)
-			return installingCondition
-		}
-		podCounter := 0
-		allPodsReady := true
-		watchingStatefulSets := []string{
-			strings.Join([]string{mco.ObjectMeta.Name, "observatorium-thanos-compact"}, "-"),
-			strings.Join([]string{mco.ObjectMeta.Name, "observatorium-thanos-receive-default"}, "-"),
-			strings.Join([]string{mco.ObjectMeta.Name, "observatorium-thanos-rule"}, "-"),
-			strings.Join([]string{mco.ObjectMeta.Name, "observatorium-thanos-store-memcached"}, "-"),
-			strings.Join([]string{mco.ObjectMeta.Name, "observatorium-thanos-store-shard-0"}, "-"),
-		}
-		statefulSetList := &appsv1.StatefulSetList{}
-		statefulSetListOpts := []client.ListOption{
-			client.InNamespace(mco.Namespace),
-		}
-		err = c.List(context.TODO(), statefulSetList, statefulSetListOpts...)
-		if err != nil {
-			log.Error(err, "Failed to list statefulSets.",
-				"MultiClusterObservability.Namespace", mco.Namespace,
-			)
-			return installingCondition
-		}
-		statefulSetCounter := 0
-		allstatefulSetReady := true
-		getResourceConditions(podList, statefulSetList, watchingPods, watchingStatefulSets,
-			&allPodsReady, &allstatefulSetReady, &podCounter, &statefulSetCounter)
-
-		if podCounter == 0 || statefulSetCounter == 0 || allPodsReady != true ||
-			podCounter < len(watchingPods) || allstatefulSetReady != true || statefulSetCounter < len(watchingStatefulSets) {
-			installingCondition = mcov1beta1.MCOCondition{
-				Type:    Installing,
-				Reason:  Installing,
-				Message: "Installation is in progress",
-			}
-		} else {
-			installingCondition = mcov1beta1.MCOCondition{
-				Type: Ready,
-			}
-		}
-	} else {
-		installingCondition = mcov1beta1.MCOCondition{
-			Type: Ready,
-		}
-	}
-
-	return installingCondition
-}
-
-func CheckS3Conf(c client.Client,
-	mco *mcov1beta1.MultiClusterObservability) *mcov1beta1.MCOCondition {
-	objStorageConf := mco.Spec.StorageConfig.MetricObjectStorage
-	secret := &corev1.Secret{}
-	namespacedName := types.NamespacedName{
-		Name:      objStorageConf.Name,
-		Namespace: mco.Namespace,
-	}
-
-	failed := mcov1beta1.MCOCondition{
-		Type:   Failed,
-		Reason: Failed,
-	}
-
-	err := c.Get(context.TODO(), namespacedName, secret)
-	if err != nil {
-		failed.Message = err.Error()
-		return &failed
-	}
-
-	data, ok := secret.Data[objStorageConf.Key]
-	if !ok {
-		failed.Message = "Failed to found the object storage configuration"
-		return &failed
-	}
-
-	ok, err = config.IsValidS3Conf(data)
-	if !ok {
-		failed.Message = err.Error()
-		return &failed
-	}
-
-	return nil
-}
-
 func getStorageClass(mco *mcov1beta1.MultiClusterObservability, cl client.Client) (string, error) {
 	storageClassSelected := mco.Spec.StorageConfig.StatefulSetStorageClass
 	// for the test, the reader is just nil
