@@ -1,13 +1,15 @@
-// Copyright (c) 2020 Red Hat, Inc.
+// Copyright (c) 2021 Red Hat, Inc.
 
 package multiclusterobservability
 
 import (
 	"bytes"
 	"context"
+	"fmt"
 
-	observatoriumv1alpha1 "github.com/observatorium/operator/api/v1alpha1"
+	observatoriumv1alpha1 "github.com/open-cluster-management/observatorium-operator/api/v1alpha1"
 	routev1 "github.com/openshift/api/route/v1"
+	appsv1 "k8s.io/api/apps/v1"
 	v1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/api/errors"
 	"k8s.io/apimachinery/pkg/api/resource"
@@ -26,8 +28,7 @@ import (
 )
 
 const (
-	obsPartoOfName = "-observatorium"
-	obsAPIGateway  = "observatorium-api"
+	obsAPIGateway = "observatorium-api"
 
 	readOnlyRoleName  = "read-only-metrics"
 	writeOnlyRoleName = "write-only-metrics"
@@ -39,7 +40,7 @@ func GenerateObservatoriumCR(
 	mco *mcov1beta1.MultiClusterObservability) (*reconcile.Result, error) {
 
 	labels := map[string]string{
-		"app": mco.Name,
+		"app": mcoconfig.GetOperandName(mcoconfig.Observatorium),
 	}
 
 	storageClassSelected, err := getStorageClass(mco, cl)
@@ -51,8 +52,8 @@ func GenerateObservatoriumCR(
 
 	observatoriumCR := &observatoriumv1alpha1.Observatorium{
 		ObjectMeta: metav1.ObjectMeta{
-			Name:      mco.Name + obsPartoOfName,
-			Namespace: mco.Namespace,
+			Name:      mcoconfig.GetOperandName(mcoconfig.Observatorium),
+			Namespace: mcoconfig.GetDefaultNamespace(),
 			Labels:    labels,
 		},
 		Spec: *newDefaultObservatoriumSpec(mco, storageClassSelected),
@@ -93,34 +94,52 @@ func GenerateObservatoriumCR(
 	oldSpecBytes, _ := yaml.Marshal(oldSpec)
 	newSpecBytes, _ := yaml.Marshal(newSpec)
 
-	if res := bytes.Compare(newSpecBytes, oldSpecBytes); res != 0 {
-		newObj := observatoriumCRFound.DeepCopy()
+	if bytes.Compare(newSpecBytes, oldSpecBytes) == 0 {
+		return nil, nil
+	}
 
-		// keep the tenant id unchanged
-		for i, newTenant := range newSpec.API.Tenants {
-			for _, oldTenant := range oldSpec.API.Tenants {
-				if oldTenant.Name == newTenant.Name && newTenant.ID != oldTenant.ID {
-					newSpec.API.Tenants[i].ID = oldTenant.ID
-					for j, hashring := range newSpec.Hashrings {
-						if util.Contains(hashring.Tenants, newTenant.ID) {
-							newSpec.Hashrings[j].Tenants = util.Remove(newSpec.Hashrings[j].Tenants, newTenant.ID)
-							newSpec.Hashrings[j].Tenants = append(newSpec.Hashrings[0].Tenants, oldTenant.ID)
-						}
-					}
-				}
-			}
-		}
-
-		newObj.Spec = newSpec
-		err = cl.Update(context.TODO(), newObj)
-		if err != nil {
-			return &reconcile.Result{}, err
+	// keep the tenant id unchanged
+	for i, newTenant := range newSpec.API.Tenants {
+		for _, oldTenant := range oldSpec.API.Tenants {
+			updateTenantID(&newSpec, newTenant, oldTenant, i)
 		}
 	}
 
+	newObj := observatoriumCRFound.DeepCopy()
+	newObj.Spec = newSpec
+	err = cl.Update(context.TODO(), newObj)
+	if err != nil {
+		return &reconcile.Result{}, err
+	}
+
+	// delete the store-share statefulset in scalein scenario
+	err = deleteStoreSts(cl, observatoriumCR.Name, *oldSpec.Store.Shards, *newSpec.Store.Shards)
+	if err != nil {
+		return &reconcile.Result{}, err
+	}
+
 	return nil, nil
 }
 
+func updateTenantID(
+	newSpec *observatoriumv1alpha1.ObservatoriumSpec,
+	newTenant observatoriumv1alpha1.APITenant,
+	oldTenant observatoriumv1alpha1.APITenant,
+	idx int) {
+
+	if oldTenant.Name == newTenant.Name && newTenant.ID == oldTenant.ID {
+		return
+	}
+
+	newSpec.API.Tenants[idx].ID = oldTenant.ID
+	for j, hashring := range newSpec.Hashrings {
+		if util.Contains(hashring.Tenants, newTenant.ID) {
+			newSpec.Hashrings[j].Tenants = util.Remove(newSpec.Hashrings[j].Tenants, newTenant.ID)
+			newSpec.Hashrings[j].Tenants = append(newSpec.Hashrings[0].Tenants, oldTenant.ID)
+		}
+	}
+}
+
 // GenerateAPIGatewayRoute defines aaa
 func GenerateAPIGatewayRoute(
 	runclient client.Client, scheme *runtime.Scheme,
@@ -129,7 +148,7 @@ func GenerateAPIGatewayRoute(
 	apiGateway := &routev1.Route{
 		ObjectMeta: metav1.ObjectMeta{
 			Name:      obsAPIGateway,
-			Namespace: mco.Namespace,
+			Namespace: mcoconfig.GetDefaultNamespace(),
 		},
 		Spec: routev1.RouteSpec{
 			Port: &routev1.RoutePort{
@@ -137,7 +156,7 @@ func GenerateAPIGatewayRoute(
 			},
 			To: routev1.RouteTargetReference{
 				Kind: "Service",
-				Name: mco.Name + "-observatorium-observatorium-api",
+				Name: mcoconfig.GetOperandName(mcoconfig.Observatorium) + "-observatorium-api",
 			},
 			TLS: &routev1.TLSConfig{
 				Termination:                   routev1.TLSTerminationPassthrough,
@@ -174,10 +193,19 @@ func newDefaultObservatoriumSpec(mco *mcov1beta1.MultiClusterObservability,
 
 	obs := &observatoriumv1alpha1.ObservatoriumSpec{}
 	obs.NodeSelector = mco.Spec.NodeSelector
+	obs.Tolerations = mco.Spec.Tolerations
 	obs.API.RBAC = newAPIRBAC()
 	obs.API.Tenants = newAPITenants()
 	obs.API.TLS = newAPITLS()
 	obs.API.Replicas = util.GetReplicaCount(mco.Spec.AvailabilityConfig, "Deployments")
+	if !mcoconfig.WithoutResourcesRequests(mco.GetAnnotations()) {
+		obs.API.Resources = v1.ResourceRequirements{
+			Requests: v1.ResourceList{
+				v1.ResourceName(v1.ResourceCPU):    resource.MustParse(mcoconfig.ObservatoriumAPICPURequets),
+				v1.ResourceName(v1.ResourceMemory): resource.MustParse(mcoconfig.ObservatoriumAPIMemoryRequets),
+			},
+		}
+	}
 	obs.Compact = newCompactSpec(mco, scSelected)
 
 	obs.Hashrings = []*observatoriumv1alpha1.Hashring{
@@ -204,14 +232,38 @@ func newDefaultObservatoriumSpec(mco *mcov1beta1.MultiClusterObservability,
 		mcoconfig.ThanosReceiveControllerImgName +
 		":" + mcoconfig.ThanosReceiveControllerImgTag
 	obs.ThanosReceiveController.Version = mcoconfig.ThanosReceiveControllerImgTag
+	if !mcoconfig.WithoutResourcesRequests(mco.GetAnnotations()) {
+		obs.ThanosReceiveController.Resources = v1.ResourceRequirements{
+			Requests: v1.ResourceList{
+				v1.ResourceName(v1.ResourceCPU):    resource.MustParse(mcoconfig.ObservatoriumReceiveControllerCPURequets),
+				v1.ResourceName(v1.ResourceMemory): resource.MustParse(mcoconfig.ObservatoriumReceiveControllerMemoryRequets),
+			},
+		}
+	}
 
 	obs.Query.Image = mcoconfig.ThanosImgRepo + "/" + mcoconfig.ThanosImgName + ":" + mcoconfig.ThanosImgTag
 	obs.Query.Version = mcoconfig.ThanosImgTag
 	obs.Query.Replicas = util.GetReplicaCount(mco.Spec.AvailabilityConfig, "Deployments")
+	if !mcoconfig.WithoutResourcesRequests(mco.GetAnnotations()) {
+		obs.Query.Resources = v1.ResourceRequirements{
+			Requests: v1.ResourceList{
+				v1.ResourceName(v1.ResourceCPU):    resource.MustParse(mcoconfig.ThanosQueryCPURequets),
+				v1.ResourceName(v1.ResourceMemory): resource.MustParse(mcoconfig.ThanosQueryMemoryRequets),
+			},
+		}
+	}
 
 	obs.QueryFrontend.Image = mcoconfig.ThanosImgRepo + "/" + mcoconfig.ThanosImgName + ":" + mcoconfig.ThanosImgTag
 	obs.QueryFrontend.Version = mcoconfig.ThanosImgTag
 	obs.QueryFrontend.Replicas = util.GetReplicaCount(mco.Spec.AvailabilityConfig, "Deployments")
+	if !mcoconfig.WithoutResourcesRequests(mco.GetAnnotations()) {
+		obs.QueryFrontend.Resources = v1.ResourceRequirements{
+			Requests: v1.ResourceList{
+				v1.ResourceName(v1.ResourceCPU):    resource.MustParse(mcoconfig.ThanosQueryFrontendCPURequets),
+				v1.ResourceName(v1.ResourceMemory): resource.MustParse(mcoconfig.ThanosQueryFrontendMemoryRequets),
+			},
+		}
+	}
 
 	replace, image := mcoconfig.ReplaceImage(mco.Annotations, obs.API.Image, mcoconfig.ObservatoriumAPIImgName)
 	if replace {
@@ -322,6 +374,14 @@ func newReceiversSpec(
 	receSpec.Replicas = util.GetReplicaCount(mco.Spec.AvailabilityConfig, "StatefulSet")
 	receSpec.ReplicationFactor = receSpec.Replicas
 	receSpec.Version = mcoconfig.ThanosImgTag
+	if !mcoconfig.WithoutResourcesRequests(mco.GetAnnotations()) {
+		receSpec.Resources = v1.ResourceRequirements{
+			Requests: v1.ResourceList{
+				v1.ResourceName(v1.ResourceCPU):    resource.MustParse(mcoconfig.ThanosReceiveCPURequets),
+				v1.ResourceName(v1.ResourceMemory): resource.MustParse(mcoconfig.ThanosReceiveMemoryRequets),
+			},
+		}
+	}
 	found, image := mcoconfig.ReplaceImage(mco.Annotations, receSpec.Image, mcoconfig.ThanosImgName)
 	if found {
 		receSpec.Image = image
@@ -338,10 +398,33 @@ func newRuleSpec(mco *mcov1beta1.MultiClusterObservability, scSelected string) o
 	ruleSpec.Image = mcoconfig.ThanosImgRepo + "/" + mcoconfig.ThanosImgName + ":" + mcoconfig.ThanosImgTag
 	ruleSpec.Replicas = util.GetReplicaCount(mco.Spec.AvailabilityConfig, "StatefulSet")
 	ruleSpec.Version = mcoconfig.ThanosImgTag
+	if !mcoconfig.WithoutResourcesRequests(mco.GetAnnotations()) {
+		ruleSpec.Resources = v1.ResourceRequirements{
+			Requests: v1.ResourceList{
+				v1.ResourceName(v1.ResourceCPU):    resource.MustParse(mcoconfig.ThanosRuleCPURequets),
+				v1.ResourceName(v1.ResourceMemory): resource.MustParse(mcoconfig.ThanosRuleMemoryRequets),
+			},
+		}
+		ruleSpec.ReloaderResources = v1.ResourceRequirements{
+			Requests: v1.ResourceList{
+				v1.ResourceName(v1.ResourceCPU):    resource.MustParse(mcoconfig.ThanosRuleReloaderCPURequets),
+				v1.ResourceName(v1.ResourceMemory): resource.MustParse(mcoconfig.ThanosRuleReloaderMemoryRequets),
+			},
+		}
+	}
+
 	found, image := mcoconfig.ReplaceImage(mco.Annotations, ruleSpec.Image, mcoconfig.ThanosImgName)
 	if found {
 		ruleSpec.Image = image
 	}
+	ruleSpec.ReloaderImage = mcoconfig.ConfigmapReloaderImgRepo + "/" +
+		mcoconfig.ConfigmapReloaderImgName + ":" + mcoconfig.ConfigmapReloaderImgTagSuffix
+	found, reloaderImage := mcoconfig.ReplaceImage(mco.Annotations,
+		mcoconfig.ConfigmapReloaderImgRepo, mcoconfig.ConfigmapReloaderKey)
+	if found {
+		ruleSpec.ReloaderImage = reloaderImage
+	}
+
 	ruleSpec.VolumeClaimTemplate = newVolumeClaimTemplate(
 		mco.Spec.StorageConfig.StatefulSetSize,
 		scSelected)
@@ -379,6 +462,14 @@ func newStoreSpec(mco *mcov1beta1.MultiClusterObservability, scSelected string)
 	storeSpec := observatoriumv1alpha1.StoreSpec{}
 	storeSpec.Image = mcoconfig.ThanosImgRepo + "/" + mcoconfig.ThanosImgName + ":" + mcoconfig.ThanosImgTag
 	storeSpec.Version = mcoconfig.ThanosImgTag
+	if !mcoconfig.WithoutResourcesRequests(mco.GetAnnotations()) {
+		storeSpec.Resources = v1.ResourceRequirements{
+			Requests: v1.ResourceList{
+				v1.ResourceName(v1.ResourceCPU):    resource.MustParse(mcoconfig.ThanosStoreCPURequets),
+				v1.ResourceName(v1.ResourceMemory): resource.MustParse(mcoconfig.ThanosStoreMemoryRequets),
+			},
+		}
+	}
 	found, image := mcoconfig.ReplaceImage(mco.Annotations, storeSpec.Image, mcoconfig.ThanosImgName)
 	if found {
 		storeSpec.Image = image
@@ -401,6 +492,20 @@ func newStoreCacheSpec(mco *mcov1beta1.MultiClusterObservability) observatoriumv
 	storeCacheSpec.ExporterImage = mcoconfig.MemcachedExporterImgRepo + "/" +
 		mcoconfig.MemcachedExporterImgName + ":" + mcoconfig.MemcachedExporterImgTag
 	storeCacheSpec.ExporterVersion = mcoconfig.MemcachedExporterImgTag
+	if !mcoconfig.WithoutResourcesRequests(mco.GetAnnotations()) {
+		storeCacheSpec.Resources = v1.ResourceRequirements{
+			Requests: v1.ResourceList{
+				v1.ResourceName(v1.ResourceCPU):    resource.MustParse(mcoconfig.ThanosCahcedCPURequets),
+				v1.ResourceName(v1.ResourceMemory): resource.MustParse(mcoconfig.ThanosCahcedMemoryRequets),
+			},
+		}
+		storeCacheSpec.ExporterResources = v1.ResourceRequirements{
+			Requests: v1.ResourceList{
+				v1.ResourceName(v1.ResourceCPU):    resource.MustParse(mcoconfig.ThanosCahcedExporterCPURequets),
+				v1.ResourceName(v1.ResourceMemory): resource.MustParse(mcoconfig.ThanosCahcedExporterMemoryRequets),
+			},
+		}
+	}
 
 	found, image := mcoconfig.ReplaceImage(mco.Annotations, storeCacheSpec.Image, mcoconfig.MemcachedImgName)
 	if found {
@@ -426,6 +531,14 @@ func newCompactSpec(mco *mcov1beta1.MultiClusterObservability, scSelected string
 	//Compactions are needed from time to time, only when new blocks appear.
 	compactSpec.Replicas = &replicas1
 	compactSpec.Version = mcoconfig.ThanosImgTag
+	if !mcoconfig.WithoutResourcesRequests(mco.GetAnnotations()) {
+		compactSpec.Resources = v1.ResourceRequirements{
+			Requests: v1.ResourceList{
+				v1.ResourceName(v1.ResourceCPU):    resource.MustParse(mcoconfig.ThanosCompactCPURequets),
+				v1.ResourceName(v1.ResourceMemory): resource.MustParse(mcoconfig.ThanosCompactMemoryRequets),
+			},
+		}
+	}
 	found, image := mcoconfig.ReplaceImage(mco.Annotations, compactSpec.Image, mcoconfig.ThanosImgName)
 	if found {
 		compactSpec.Image = image
@@ -467,3 +580,26 @@ func mergeVolumeClaimTemplate(oldVolumn,
 	}
 	return oldVolumn
 }
+
+func deleteStoreSts(cl client.Client, name string, oldNum int32, newNum int32) error {
+	if oldNum > newNum {
+		for i := newNum; i < oldNum; i++ {
+			stsName := fmt.Sprintf("%s-thanos-store-shard-%d", name, i)
+			found := &appsv1.StatefulSet{}
+			err := cl.Get(context.TODO(), types.NamespacedName{Name: stsName, Namespace: mcoconfig.GetDefaultNamespace()}, found)
+			if err != nil {
+				if !errors.IsNotFound(err) {
+					log.Error(err, "Failed to get statefulset", "name", stsName)
+					return err
+				}
+			} else {
+				err = cl.Delete(context.TODO(), found)
+				if err != nil {
+					log.Error(err, "Failed to delete statefulset", "name", stsName)
+					return err
+				}
+			}
+		}
+	}
+	return nil
+}
