@@ -1,10 +1,9 @@
-// Copyright (c) 2020 Red Hat, Inc.
+// Copyright (c) 2021 Red Hat, Inc.
 
 package placementrule
 
 import (
 	"context"
-	"fmt"
 	"os"
 
 	certv1alpha1 "github.com/jetstack/cert-manager/pkg/apis/certmanager/v1alpha1"
@@ -28,7 +27,7 @@ import (
 	"sigs.k8s.io/controller-runtime/pkg/reconcile"
 	"sigs.k8s.io/controller-runtime/pkg/source"
 
-	addonv1alpha1 "github.com/open-cluster-management/addon-framework/api/v1alpha1"
+	addonv1alpha1 "github.com/open-cluster-management/api/addon/v1alpha1"
 	workv1 "github.com/open-cluster-management/api/work/v1"
 	appsv1 "github.com/open-cluster-management/multicloud-operators-placementrule/pkg/apis/apps/v1"
 	mcov1beta1 "github.com/open-cluster-management/multicluster-monitoring-operator/pkg/apis/observability/v1beta1"
@@ -86,7 +85,7 @@ func add(mgr manager.Manager, r reconcile.Reconciler) error {
 		return err
 	}
 
-	name := config.GetPlacementRuleName()
+	name := config.GetDefaultCRName()
 
 	pred := predicate.Funcs{
 		CreateFunc: func(e event.CreateEvent) bool {
@@ -96,7 +95,9 @@ func add(mgr manager.Manager, r reconcile.Reconciler) error {
 			return false
 		},
 		UpdateFunc: func(e event.UpdateEvent) bool {
-			if e.MetaNew.GetName() == name && e.MetaNew.GetNamespace() == watchNamespace {
+			if e.MetaNew.GetName() == name &&
+				e.MetaNew.GetNamespace() == watchNamespace &&
+				e.MetaNew.GetResourceVersion() != e.MetaOld.GetResourceVersion() {
 				return true
 			}
 			return false
@@ -125,82 +126,36 @@ func add(mgr manager.Manager, r reconcile.Reconciler) error {
 			}
 		})
 
-	// Only handle delete event for observabilityaddon
-	epPred := predicate.Funcs{
-		CreateFunc: func(e event.CreateEvent) bool {
-			return false
-		},
-		UpdateFunc: func(e event.UpdateEvent) bool {
-			return false
-		},
-		DeleteFunc: func(e event.DeleteEvent) bool {
-			if e.Meta.GetName() == epConfigName && e.Meta.GetLabels()[ownerLabelKey] == ownerLabelValue {
-				return true
-			}
-			return false
-		},
-	}
-
 	// secondary watch for observabilityaddon
-	err = c.Watch(&source.Kind{Type: &mcov1beta1.ObservabilityAddon{}},
-		&handler.EnqueueRequestsFromMapFunc{
-			ToRequests: mapFn,
-		},
-		epPred)
+	err = watchObservabilityaddon(c, mapFn)
 	if err != nil {
 		return err
 	}
 
+	// secondary watch for manifestwork
 	gk := schema.GroupKind{Group: workv1.GroupVersion.Group, Kind: "ManifestWork"}
 	_, err = r.(*ReconcilePlacementRule).restMapper.RESTMapping(gk, workv1.GroupVersion.Version)
 	if err == nil {
-		workPred := predicate.Funcs{
-			CreateFunc: func(e event.CreateEvent) bool {
-				return false
-			},
-			UpdateFunc: func(e event.UpdateEvent) bool {
-				if e.MetaNew.GetName() == workName && e.MetaNew.GetLabels()[ownerLabelKey] == ownerLabelValue {
-					return true
-				}
-				return false
-			},
-			DeleteFunc: func(e event.DeleteEvent) bool {
-				if e.Meta.GetName() == workName && e.Meta.GetLabels()[ownerLabelKey] == ownerLabelValue {
-					return true
-				}
-				return false
-			},
-		}
-
-		// secondary watch for manifestwork
-		err = c.Watch(&source.Kind{Type: &workv1.ManifestWork{}},
-			&handler.EnqueueRequestsFromMapFunc{
-				ToRequests: mapFn,
-			},
-			workPred)
+		err = watchManifestwork(c, mapFn)
 		if err != nil {
 			return err
 		}
 	}
 
-	mcoPred := predicate.Funcs{
-		CreateFunc: func(e event.CreateEvent) bool {
-			return true
-		},
-		UpdateFunc: func(e event.UpdateEvent) bool {
-			return false
-		},
-		DeleteFunc: func(e event.DeleteEvent) bool {
-			return true
-		},
+	// secondary watch for mco
+	err = watchMCO(c, mapFn)
+	if err != nil {
+		return err
 	}
 
-	// secondary watch for mco
-	err = c.Watch(&source.Kind{Type: &mcov1beta1.MultiClusterObservability{}},
-		&handler.EnqueueRequestsFromMapFunc{
-			ToRequests: mapFn,
-		},
-		mcoPred)
+	// secondary watch for custom whitelist configmap
+	err = watchWhitelistCM(c, mapFn)
+	if err != nil {
+		return err
+	}
+
+	// secondary watch for certificate secrets
+	err = watchCertficate(c, mapFn)
 	if err != nil {
 		return err
 	}
@@ -209,7 +164,21 @@ func add(mgr manager.Manager, r reconcile.Reconciler) error {
 	gk = schema.GroupKind{Group: ocinfrav1.GroupVersion.Group, Kind: "APIServer"}
 	_, err = r.(*ReconcilePlacementRule).restMapper.RESTMapping(gk, ocinfrav1.GroupVersion.Version)
 	if err == nil {
-		err = watchAPIServerForKubeconfig(c)
+		pred = predicate.Funcs{
+			CreateFunc: func(e event.CreateEvent) bool {
+				return false
+			},
+			UpdateFunc: func(e event.UpdateEvent) bool {
+				if e.MetaNew.GetResourceVersion() != e.MetaOld.GetResourceVersion() {
+					return true
+				}
+				return false
+			},
+			DeleteFunc: func(e event.DeleteEvent) bool {
+				return false
+			},
+		}
+		err = c.Watch(&source.Kind{Type: &ocinfrav1.APIServer{}}, &handler.EnqueueRequestForObject{}, pred)
 		if err != nil {
 			return err
 		}
@@ -241,7 +210,8 @@ func (r *ReconcilePlacementRule) Reconcile(request reconcile.Request) (reconcile
 	reqLogger.Info("Reconciling PlacementRule")
 
 	if config.GetMonitoringCRName() == "" {
-		return reconcile.Result{}, fmt.Errorf("multicluster observability resource is not available")
+		reqLogger.Info("multicluster observability resource is not available")
+		return reconcile.Result{}, nil
 	}
 
 	// Fetch the MultiClusterObservability instance
@@ -274,106 +244,27 @@ func (r *ReconcilePlacementRule) Reconcile(request reconcile.Request) (reconcile
 	opts := &client.ListOptions{
 		LabelSelector: labels.SelectorFromSet(map[string]string{ownerLabelKey: ownerLabelValue}),
 	}
-	epList := &mcov1beta1.ObservabilityAddonList{}
-	err = r.client.List(context.TODO(), epList, opts)
+	obsAddonList := &mcov1beta1.ObservabilityAddonList{}
+	err = r.client.List(context.TODO(), obsAddonList, opts)
 	if err != nil {
 		reqLogger.Error(err, "Failed to list observabilityaddon resource")
 		return reconcile.Result{}, err
 	}
+
 	if !deleteAll {
-		// create the clusterrole if not there
-		if !isCRoleCreated {
-			err = createClusterRole(r.client)
-
-			if err != nil {
-				return reconcile.Result{}, err
-			}
-			isCRoleCreated = true
-		}
-		//Check if ClusterManagementAddon is created or create it
-		if !isClusterManagementAddonCreated {
-			err := util.CreateClusterManagementAddon(r.client)
-			if err != nil {
-				return reconcile.Result{}, err
-			}
-			isClusterManagementAddonCreated = true
-		}
-
-		imagePullSecret := &corev1.Secret{}
-		err = r.client.Get(context.TODO(),
-			types.NamespacedName{
-				Name:      mco.Spec.ImagePullSecret,
-				Namespace: request.Namespace,
-			}, imagePullSecret)
+		res, err := createAllRelatedRes(r.client, r.restMapper, request, mco, obsAddonList)
 		if err != nil {
-			if errors.IsNotFound(err) {
-				imagePullSecret = nil
-			} else {
-				// Error reading the object - requeue the request.
-				return reconcile.Result{}, err
-			}
-		}
-		mco.Namespace = watchNamespace
-		// Fetch the PlacementRule instance
-		instance := &appsv1.PlacementRule{}
-		err = r.client.Get(context.TODO(), request.NamespacedName, instance)
-		if err != nil {
-			if errors.IsNotFound(err) {
-				// Request object not found, could have been deleted after reconcile request.
-				// Owned objects are automatically garbage collected. For additional cleanup logic use finalizers.
-				// Return and don't requeue
-				return reconcile.Result{}, nil
-			}
-			// Error reading the object - requeue the request.
-			return reconcile.Result{}, err
-		}
-
-		currentClusters := []string{}
-		for _, ep := range epList.Items {
-			currentClusters = append(currentClusters, ep.Namespace)
-		}
-
-		for _, decision := range instance.Status.Decisions {
-			reqLogger.Info("Monitoring operator should be installed in cluster", "cluster_name", decision.ClusterName)
-			currentClusters = util.Remove(currentClusters, decision.ClusterNamespace)
-			err = createManagedClusterRes(r.client, r.restMapper, mco, imagePullSecret,
-				decision.ClusterName, decision.ClusterNamespace)
-			if err != nil {
-				return reconcile.Result{}, err
-			}
-		}
-
-		for _, cluster := range currentClusters {
-			reqLogger.Info("To delete observabilityAddon", "namespace", cluster)
-			err = deleteEndpointConfigCR(r.client, cluster)
-			if err != nil {
-				reqLogger.Error(err, "Failed to delete observabilityaddon", "namespace", cluster)
-				return reconcile.Result{}, err
-			}
+			return res, err
 		}
 	} else {
-		for _, ep := range epList.Items {
-			err = deleteEndpointConfigCR(r.client, ep.Namespace)
-			if err != nil {
-				reqLogger.Error(err, "Failed to delete observabilityaddon", "namespace", ep.Namespace)
-				return reconcile.Result{}, err
-			}
-		}
-		err = deleteClusterRole(r.client)
+		res, err := deleteAllRelatedRes(r.client, obsAddonList)
 		if err != nil {
-			return reconcile.Result{}, err
+			return res, err
 		}
-		isCRoleCreated = false
-		//delete ClusterManagementAddon
-		err = deleteClusterManagementAddon(r.client)
-		if err != nil {
-			return reconcile.Result{}, err
-		}
-		isClusterManagementAddonCreated = false
 	}
 
-	epList = &mcov1beta1.ObservabilityAddonList{}
-	err = r.client.List(context.TODO(), epList, opts)
+	obsAddonList = &mcov1beta1.ObservabilityAddonList{}
+	err = r.client.List(context.TODO(), obsAddonList, opts)
 	if err != nil {
 		reqLogger.Error(err, "Failed to list observabilityaddon resource")
 		return reconcile.Result{}, err
@@ -385,8 +276,10 @@ func (r *ReconcilePlacementRule) Reconcile(request reconcile.Request) (reconcile
 		return reconcile.Result{}, err
 	}
 	latestClusters := []string{}
-	for _, ep := range epList.Items {
-		latestClusters = append(latestClusters, ep.Namespace)
+	staleAddons := []string{}
+	for _, addon := range obsAddonList.Items {
+		latestClusters = append(latestClusters, addon.Namespace)
+		staleAddons = append(staleAddons, addon.Namespace)
 	}
 	for _, work := range workList.Items {
 		if !util.Contains(latestClusters, work.Namespace) {
@@ -395,9 +288,127 @@ func (r *ReconcilePlacementRule) Reconcile(request reconcile.Request) (reconcile
 			if err != nil {
 				return reconcile.Result{}, err
 			}
+		} else {
+			staleAddons = util.Remove(staleAddons, work.Namespace)
 		}
 	}
 
+	// delete stale addons if manifestwork does not exist
+	for _, addon := range staleAddons {
+		err = deleteStaleObsAddon(r.client, addon)
+		if err != nil {
+			return reconcile.Result{}, err
+		}
+	}
+
+	return reconcile.Result{}, nil
+}
+
+func createAllRelatedRes(
+	client client.Client,
+	restMapper meta.RESTMapper,
+	request reconcile.Request,
+	mco *mcov1beta1.MultiClusterObservability,
+	obsAddonList *mcov1beta1.ObservabilityAddonList) (reconcile.Result, error) {
+
+	// create the clusterrole if not there
+	if !isCRoleCreated {
+		err := createClusterRole(client)
+
+		if err != nil {
+			return reconcile.Result{}, err
+		}
+		isCRoleCreated = true
+	}
+	//Check if ClusterManagementAddon is created or create it
+	if !isClusterManagementAddonCreated {
+		err := util.CreateClusterManagementAddon(client)
+		if err != nil {
+			return reconcile.Result{}, err
+		}
+		isClusterManagementAddonCreated = true
+	}
+
+	imagePullSecret := &corev1.Secret{}
+	err := client.Get(context.TODO(),
+		types.NamespacedName{
+			Name:      mco.Spec.ImagePullSecret,
+			Namespace: request.Namespace,
+		}, imagePullSecret)
+	if err != nil {
+		if errors.IsNotFound(err) {
+			imagePullSecret = nil
+		} else {
+			// Error reading the object - requeue the request.
+			return reconcile.Result{}, err
+		}
+	}
+	mco.Namespace = watchNamespace
+	// Fetch the PlacementRule instance
+	instance := &appsv1.PlacementRule{}
+	err = client.Get(context.TODO(),
+		types.NamespacedName{
+			Name:      config.GetDefaultCRName(),
+			Namespace: config.GetDefaultNamespace()},
+		instance)
+	if err != nil {
+		if errors.IsNotFound(err) {
+			// Request object not found, could have been deleted after reconcile request.
+			// Owned objects are automatically garbage collected. For additional cleanup logic use finalizers.
+			// Return and don't requeue
+			return reconcile.Result{}, nil
+		}
+		// Error reading the object - requeue the request.
+		return reconcile.Result{}, err
+	}
+
+	currentClusters := []string{}
+	for _, ep := range obsAddonList.Items {
+		currentClusters = append(currentClusters, ep.Namespace)
+	}
+
+	for _, decision := range instance.Status.Decisions {
+		log.Info("Monitoring operator should be installed in cluster", "cluster_name", decision.ClusterName)
+		currentClusters = util.Remove(currentClusters, decision.ClusterNamespace)
+		err = createManagedClusterRes(client, restMapper, mco, imagePullSecret,
+			decision.ClusterName, decision.ClusterNamespace)
+		if err != nil {
+			return reconcile.Result{}, err
+		}
+	}
+
+	for _, cluster := range currentClusters {
+		log.Info("To delete observabilityAddon", "namespace", cluster)
+		err = deleteObsAddon(client, cluster)
+		if err != nil {
+			log.Error(err, "Failed to delete observabilityaddon", "namespace", cluster)
+			return reconcile.Result{}, err
+		}
+	}
+	return reconcile.Result{}, nil
+}
+
+func deleteAllRelatedRes(
+	client client.Client,
+	obsAddonList *mcov1beta1.ObservabilityAddonList) (reconcile.Result, error) {
+	for _, ep := range obsAddonList.Items {
+		err := deleteObsAddon(client, ep.Namespace)
+		if err != nil {
+			log.Error(err, "Failed to delete observabilityaddon", "namespace", ep.Namespace)
+			return reconcile.Result{}, err
+		}
+	}
+	err := deleteClusterRole(client)
+	if err != nil {
+		return reconcile.Result{}, err
+	}
+	isCRoleCreated = false
+	//delete ClusterManagementAddon
+	err = util.DeleteClusterManagementAddon(client)
+	if err != nil {
+		return reconcile.Result{}, err
+	}
+	isClusterManagementAddonCreated = false
 	return reconcile.Result{}, nil
 }
 
@@ -414,7 +425,7 @@ func createManagedClusterRes(client client.Client, restMapper meta.RESTMapper,
 		return err
 	}
 
-	err = createEndpointConfigCR(client, namespace)
+	err = createObsAddon(client, namespace)
 	if err != nil {
 		log.Error(err, "Failed to create observabilityaddon")
 		return err
@@ -477,22 +488,162 @@ func deleteManagedClusterRes(client client.Client, namespace string) error {
 	return nil
 }
 
-func watchAPIServerForKubeconfig(c controller.Controller) error {
-	// watch APIServer for kubeconfig
-	pred := predicate.Funcs{
+func watchObservabilityaddon(c controller.Controller, mapFn handler.ToRequestsFunc) error {
+	// Only handle delete event for observabilityaddon
+	epPred := predicate.Funcs{
 		CreateFunc: func(e event.CreateEvent) bool {
 			return false
 		},
+		UpdateFunc: func(e event.UpdateEvent) bool {
+			return false
+		},
+		DeleteFunc: func(e event.DeleteEvent) bool {
+			if e.Meta.GetName() == obsAddonName &&
+				e.Meta.GetLabels()[ownerLabelKey] == ownerLabelValue {
+				return true
+			}
+			return false
+		},
+	}
+
+	err := c.Watch(&source.Kind{Type: &mcov1beta1.ObservabilityAddon{}},
+		&handler.EnqueueRequestsFromMapFunc{
+			ToRequests: mapFn,
+		},
+		epPred)
+	if err != nil {
+		return err
+	}
+	return nil
+}
+
+func watchManifestwork(c controller.Controller, mapFn handler.ToRequestsFunc) error {
+	workPred := predicate.Funcs{
+		CreateFunc: func(e event.CreateEvent) bool {
+			return false
+		},
+		UpdateFunc: func(e event.UpdateEvent) bool {
+			if e.MetaNew.GetName() == workName &&
+				e.MetaNew.GetLabels()[ownerLabelKey] == ownerLabelValue &&
+				e.MetaNew.GetResourceVersion() != e.MetaOld.GetResourceVersion() {
+				return true
+			}
+			return false
+		},
+		DeleteFunc: func(e event.DeleteEvent) bool {
+			if e.Meta.GetName() == workName && e.Meta.GetLabels()[ownerLabelKey] == ownerLabelValue {
+				return true
+			}
+			return false
+		},
+	}
+
+	err := c.Watch(&source.Kind{Type: &workv1.ManifestWork{}},
+		&handler.EnqueueRequestsFromMapFunc{
+			ToRequests: mapFn,
+		},
+		workPred)
+	if err != nil {
+		return err
+	}
+	return nil
+}
+
+func watchWhitelistCM(c controller.Controller, mapFn handler.ToRequestsFunc) error {
+	customWhitelistPred := predicate.Funcs{
+		CreateFunc: func(e event.CreateEvent) bool {
+			if e.Meta.GetName() == config.WhitelistCustomConfigMapName &&
+				e.Meta.GetNamespace() == config.GetDefaultNamespace() {
+				return true
+			}
+			return false
+		},
+		UpdateFunc: func(e event.UpdateEvent) bool {
+			if e.MetaNew.GetName() == config.WhitelistCustomConfigMapName &&
+				e.MetaNew.GetNamespace() == config.GetDefaultNamespace() &&
+				e.MetaNew.GetResourceVersion() != e.MetaOld.GetResourceVersion() {
+				return true
+			}
+			return false
+		},
+		DeleteFunc: func(e event.DeleteEvent) bool {
+			if e.Meta.GetName() == config.WhitelistCustomConfigMapName &&
+				e.Meta.GetNamespace() == config.GetDefaultNamespace() {
+				return true
+			}
+			return false
+		},
+	}
+
+	err := c.Watch(&source.Kind{Type: &corev1.ConfigMap{}},
+		&handler.EnqueueRequestsFromMapFunc{
+			ToRequests: mapFn,
+		},
+		customWhitelistPred)
+	if err != nil {
+		return err
+	}
+	return nil
+}
+
+func watchMCO(c controller.Controller, mapFn handler.ToRequestsFunc) error {
+	mcoPred := predicate.Funcs{
+		CreateFunc: func(e event.CreateEvent) bool {
+			return true
+		},
 		UpdateFunc: func(e event.UpdateEvent) bool {
 			if e.MetaNew.GetResourceVersion() != e.MetaOld.GetResourceVersion() {
 				return true
 			}
 			return false
 		},
+		DeleteFunc: func(e event.DeleteEvent) bool {
+			return true
+		},
+	}
+
+	err := c.Watch(&source.Kind{Type: &mcov1beta1.MultiClusterObservability{}},
+		&handler.EnqueueRequestsFromMapFunc{
+			ToRequests: mapFn,
+		},
+		mcoPred)
+	if err != nil {
+		return err
+	}
+	return nil
+}
+
+func watchCertficate(c controller.Controller, mapFn handler.ToRequestsFunc) error {
+	customWhitelistPred := predicate.Funcs{
+		CreateFunc: func(e event.CreateEvent) bool {
+			if e.Meta.GetName() == certsName ||
+				e.Meta.GetName() == config.ServerCerts &&
+					e.Meta.GetNamespace() == config.GetDefaultNamespace() {
+				return true
+			}
+			return false
+		},
+		UpdateFunc: func(e event.UpdateEvent) bool {
+			if (e.MetaNew.GetName() == certsName ||
+				e.MetaNew.GetName() == config.ServerCerts &&
+					e.MetaNew.GetNamespace() == config.GetDefaultNamespace()) &&
+				e.MetaNew.GetResourceVersion() != e.MetaOld.GetResourceVersion() {
+				return true
+			}
+			return false
+		},
 		DeleteFunc: func(e event.DeleteEvent) bool {
 			return false
 		},
 	}
 
-	return c.Watch(&source.Kind{Type: &ocinfrav1.APIServer{}}, &handler.EnqueueRequestForObject{}, pred)
+	err := c.Watch(&source.Kind{Type: &corev1.Secret{}},
+		&handler.EnqueueRequestsFromMapFunc{
+			ToRequests: mapFn,
+		},
+		customWhitelistPred)
+	if err != nil {
+		return err
+	}
+	return nil
 }
